import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.sql.types.StructType;
import org.bson.Document;

public class SparkConsumer {

    public static void main(String[] args) throws TimeoutException, StreamingQueryException {

        //Create and get Spark session
        SparkSession spark = getSparkSession();

        //Read data from Kafka
        Dataset<Row> rawJsonDataFrame = loadDatafromKafka(spark);

        Dataset<Row> df = convertingJsonDatasetToSchemaDataset(rawJsonDataFrame);

        //Filtering dataset based on visa cardType
        Dataset<Row> filteredDataFrame = df.filter(functions.col("cardType").equalTo("Visa"));

        //Converting DataFrame to Dataset
        Dataset<UserCardInfo> userCardInfoDataset = filteredDataFrame.as(Encoders.bean(UserCardInfo.class));

        //Writing data to mongo db
        writeDataToMongoDB(userCardInfoDataset);

    }

    private static Dataset<Row> convertingJsonDatasetToSchemaDataset(Dataset<Row> rawItemDF) {
        StructType structType = Encoders.bean(UserCardInfo.class).schema();

        Dataset<Row> df = rawItemDF
                .selectExpr("CAST(value as String) as message")
                .select(functions.from_json(functions.col("message"), structType).as("data"))
                .select("data.*")
                .repartition(400);
        return df;
    }

    private static void writeDataToMongoDB(Dataset<UserCardInfo> ds)
            throws TimeoutException, StreamingQueryException {
        StreamingQuery query = ds.writeStream().outputMode(OutputMode.Append())
                .option("checkpointLocation", "checkpointlocaion/streamingjob")
                .foreach(new ForeachWriter<UserCardInfo>() {
                    private MongoClient mongoClient;
                    private MongoDatabase database;
                    private MongoCollection<Document> collection;

                    @Override
                    public boolean open(long partitionId, long epochId) {
                        mongoClient = MongoClients.create("mongodb://localhost:27017");
                        database = mongoClient.getDatabase("userdb");
                        collection = database.getCollection("user_card_details");
                        return true;
                    }

                    @Override
                    public void process(UserCardInfo userCardInfo) {
                        // Create a document with the object data
                        Document document = new Document();
                        document.append("userId", userCardInfo.getUserId());
                        document.append("userName", userCardInfo.getUserName());
                        document.append("regMobileNo", userCardInfo.getRegMobileNo());
                        document.append("gender", userCardInfo.getGender());
                        document.append("cardId", userCardInfo.getCardId());
                        document.append("cardType", userCardInfo.getCardType());
                        document.append("cardNumber", userCardInfo.getCardNumber());
                        document.append("cardStatus", userCardInfo.getCardStatus());

                        // Insert the document into the collection
                        collection.insertOne(document);
                    }

                    @Override
                    public void close(Throwable errorOrNull) {
                        // Close the MongoDB connection
                        mongoClient.close();
                    }
                }).start();
        query.awaitTermination();
    }

    private static Dataset<Row> loadDatafromKafka(SparkSession spark) {
        Dataset<Row> rawDataFrame = spark
                .readStream()
                .format("kafka")
                .options(constructKafkaConfigMap())
                .load();
        return rawDataFrame;
    }

    private static Map<String, String> constructKafkaConfigMap() {
        Map<String, String> kafkaConfigMap = new HashMap<>();
        kafkaConfigMap.put("kafka.bootstrap.servers", "localhost:9092");
        kafkaConfigMap.put("subscribe", "UserInfoTopic");
        kafkaConfigMap.put("startingOffsets", "earliest");
        kafkaConfigMap.put("failOnDataLoss", "false");
        kafkaConfigMap.put("kafka.con.group.id", "local");
        return kafkaConfigMap;
    }

    private static SparkSession getSparkSession() {
        SparkSession spark = SparkSession.builder()
                .master("local")
                .appName("SparkConsumer")
                .config("spark.mongodb.output.uri", "mongodb://127.0.0.1/userdb.users_info")
                .getOrCreate();
        return spark;
    }
}
