import com.mongodb.spark.config.ReadConfig;
import java.util.concurrent.TimeoutException;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.functions;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.storage.StorageLevel;

public class SparkProducer {


    public static void main(String[] args) throws TimeoutException, StreamingQueryException {

        //Create and get Spark Session
        SparkSession spark = getSparkSession();

        //Load userInfo data from mongoDB
        Dataset<Row> datasetUserInfo = loadUserInfoFromMongoDB(spark);

        //Load cardInfo data from mongoDB
        Dataset<Row> datasetUserCardInfo = loadUserCardInfoFromDB(spark);

        //Combining user and card info using spark sql
        Dataset<Row> userCardInfo = joiningUserAndCardInfo(spark);

        //Converting combined dataset into json
        Dataset<Row> userCardInfoJson = convertToJsonDataFrame(userCardInfo);

        //Publish converted json dataset to kafka
        publishToKaka(userCardInfoJson);

        //Closing spark session
        spark.close();

    }

    private static void publishToKaka(Dataset<Row> userCardInfoJson) {
        userCardInfoJson
                .write()
                .format("kafka")
                .option("kafka.bootstrap.servers", "localhost:9092")
                .option("topic", "UserInfoTopic").save();
    }

    private static Dataset<Row> convertToJsonDataFrame(Dataset<Row> userCardInfo) {
        Dataset<Row> userCardInfoJson = userCardInfo
                .withColumn("value", functions.to_json(functions.struct(functions.col("gender"),
                        functions.col("regMobileNo"), functions.col("userName"),
                        functions.col("userId"), functions.col("cardId"),
                        functions.col("cardNumber"), functions.col("cardStatus"),
                        functions.col("cardType")))).alias("value").select("value")
                .persist(StorageLevel.MEMORY_AND_DISK());
        userCardInfoJson.show(false);
        return userCardInfoJson;
    }

    private static Dataset<Row> joiningUserAndCardInfo(SparkSession spark) {
        Dataset<Row> userCardInfo = spark
                .sql("SELECT user.gender,user.regMobileNo,user.userName,card.userId as userId,card._id as cardId,card.cardNumber,card.cardStatus,card.cardType FROM user JOIN card ON user._id = card.userId");
        userCardInfo.show();
        return userCardInfo;
    }

    private static Dataset<Row> loadUserCardInfoFromDB(SparkSession spark) {
        ReadConfig readConfigUserCardInfo = ReadConfig.create(spark)
                .withOption("uri", "mongodb://localhost:27017")
                .withOption("database", "userdb")
                .withOption("collection", "users_cards_info");

        Dataset<Row> datasetUserCardInfo = spark.read()
                .format("mongo")
                .options(readConfigUserCardInfo.asOptions())
                .load();
        datasetUserCardInfo.createOrReplaceTempView("card");
        datasetUserCardInfo.show();
        return datasetUserCardInfo;
    }

    private static Dataset<Row> loadUserInfoFromMongoDB(SparkSession spark) {
        ReadConfig readConfigUserInfo = ReadConfig.create(spark)
                .withOption("uri", "mongodb://localhost:27017")
                .withOption("database", "userdb")
                .withOption("collection", "users_info");

        Dataset<Row> datasetUserInfo = spark.read()
                .format("mongo")
                .options(readConfigUserInfo.asOptions())
                .load();
        datasetUserInfo.createOrReplaceTempView("user");
        datasetUserInfo.show();
        return datasetUserInfo;
    }

    private static SparkSession getSparkSession() {
        SparkSession spark = SparkSession.builder()
                .master("local")
                .appName("SparkProducer")
                .config("spark.mongodb.input.uri", "mongodb://127.0.0.1/userdb.users_info")
                .getOrCreate();
        return spark;
    }
}

